@@ -34,13 +34,9 @@
 **  modified to work with our data types.
 */
 
-#ifndef HAVE_WIN32_CONFIG_H
 #include "xmlrpc_config.h"
-#else
-#include "xmlrpc_win32_config.h"
-#endif
 
-#include "xmlrpc.h"
+#include "xmlrpc-c/base.h"
 
 #define CRLF    "\015\012"
 #define CR      '\015'
@@ -79,18 +75,18 @@ PERFORMANCE OF THIS SOFTWARE.
 ******************************************************************/
 
 static char table_a2b_base64[] = {
-	-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
-	-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
-	-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, -1,-1,-1,63,
-	52,53,54,55, 56,57,58,59, 60,61,-1,-1, -1, 0,-1,-1, /* Note PAD->0 */
-	-1, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
-	15,16,17,18, 19,20,21,22, 23,24,25,-1, -1,-1,-1,-1,
-	-1,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
-	41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
+    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
+    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
+    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, -1,-1,-1,63,
+    52,53,54,55, 56,57,58,59, 60,61,-1,-1, -1, 0,-1,-1, /* Note PAD->0 */
+    -1, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
+    15,16,17,18, 19,20,21,22, 23,24,25,-1, -1,-1,-1,-1,
+    -1,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
+    41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
 };
 
 #define BASE64_PAD '='
-#define BASE64_MAXBIN 57	/* Max binary chunk size (76 char line) */
+#define BASE64_MAXBIN 57    /* Max binary chunk size (76 char line) */
 #define BASE64_LINE_SZ 128      /* Buffer size for a single line. */    
 
 static unsigned char table_b2a_base64[] =
@@ -98,9 +94,9 @@ static unsigned char table_b2a_base64[] =
 
 static xmlrpc_mem_block *
 xmlrpc_base64_encode_internal (xmlrpc_env *env,
-			       unsigned char *bin_data,
-			       size_t bin_len,
-			       int want_newlines)
+                               unsigned char *bin_data,
+                               size_t bin_len,
+                               int want_newlines)
 {
     size_t chunk_start, chunk_left;
     unsigned char *ascii_data;
@@ -116,60 +112,60 @@ xmlrpc_base64_encode_internal (xmlrpc_env *env,
 
     /* Deal with empty data blocks gracefully. Yuck. */
     if (bin_len == 0) {
-	if (want_newlines)
-	    XMLRPC_TYPED_MEM_BLOCK_APPEND(char, env, output, CRLF, 2);
-	goto cleanup;
+        if (want_newlines)
+            XMLRPC_TYPED_MEM_BLOCK_APPEND(char, env, output, CRLF, 2);
+        goto cleanup;
     }
 
     /* Process our binary data in line-sized chunks. */
     for (chunk_start=0; chunk_start < bin_len; chunk_start += BASE64_MAXBIN) {
 
-	/* Set up our per-line state. */
-	ascii_data = &line_buffer[0];
-	chunk_left = bin_len - chunk_start;
-	if (chunk_left > BASE64_MAXBIN)
-	    chunk_left = BASE64_MAXBIN;
-	leftbits = 0;
-	leftchar = 0;
+        /* Set up our per-line state. */
+        ascii_data = &line_buffer[0];
+        chunk_left = bin_len - chunk_start;
+        if (chunk_left > BASE64_MAXBIN)
+            chunk_left = BASE64_MAXBIN;
+        leftbits = 0;
+        leftchar = 0;
 
-	for(; chunk_left > 0; chunk_left--, bin_data++) {
-	    /* Shift the data into our buffer */
-	    leftchar = (leftchar << 8) | *bin_data;
-	    leftbits += 8;
+        for(; chunk_left > 0; chunk_left--, bin_data++) {
+            /* Shift the data into our buffer */
+            leftchar = (leftchar << 8) | *bin_data;
+            leftbits += 8;
 
-	    /* See if there are 6-bit groups ready */
-	    while (leftbits >= 6) {
-		this_ch = (leftchar >> (leftbits-6)) & 0x3f;
-		leftbits -= 6;
-		*ascii_data++ = table_b2a_base64[this_ch];
-	    }
-	}
-	if (leftbits == 2) {
-	    *ascii_data++ = table_b2a_base64[(leftchar&3) << 4];
-	    *ascii_data++ = BASE64_PAD;
-	    *ascii_data++ = BASE64_PAD;
-	} else if (leftbits == 4) {
-	    *ascii_data++ = table_b2a_base64[(leftchar&0xf) << 2];
-	    *ascii_data++ = BASE64_PAD;
-	} 
+            /* See if there are 6-bit groups ready */
+            while (leftbits >= 6) {
+                this_ch = (leftchar >> (leftbits-6)) & 0x3f;
+                leftbits -= 6;
+                *ascii_data++ = table_b2a_base64[this_ch];
+            }
+        }
+        if (leftbits == 2) {
+            *ascii_data++ = table_b2a_base64[(leftchar&3) << 4];
+            *ascii_data++ = BASE64_PAD;
+            *ascii_data++ = BASE64_PAD;
+        } else if (leftbits == 4) {
+            *ascii_data++ = table_b2a_base64[(leftchar&0xf) << 2];
+            *ascii_data++ = BASE64_PAD;
+        } 
 
-	/* Append a courtesy CRLF. */
-	if (want_newlines) {
-	    *ascii_data++ = CR;
-	    *ascii_data++ = LF;
-	}
-	
-	/* Save our line. */
-	XMLRPC_TYPED_MEM_BLOCK_APPEND(char, env, output, line_buffer,
-				      ascii_data - &line_buffer[0]);
-	XMLRPC_FAIL_IF_FAULT(env);
+        /* Append a courtesy CRLF. */
+        if (want_newlines) {
+            *ascii_data++ = CR;
+            *ascii_data++ = LF;
+        }
+    
+        /* Save our line. */
+        XMLRPC_TYPED_MEM_BLOCK_APPEND(char, env, output, line_buffer,
+                                      ascii_data - &line_buffer[0]);
+        XMLRPC_FAIL_IF_FAULT(env);
     }
 
  cleanup:
     if (env->fault_occurred) {
-	if (output)
-	    xmlrpc_mem_block_free(output);
-	return NULL;
+        if (output)
+            xmlrpc_mem_block_free(output);
+        return NULL;
     }
     return output;
 }
@@ -184,18 +180,18 @@ xmlrpc_base64_encode (xmlrpc_env *env, unsigned char *bin_data, size_t bin_len)
 
 xmlrpc_mem_block *
 xmlrpc_base64_encode_without_newlines (xmlrpc_env *env,
-				       unsigned char *bin_data,
-				       size_t bin_len)
+                                       unsigned char *bin_data,
+                                       size_t bin_len)
 {
     return xmlrpc_base64_encode_internal(env, bin_data, bin_len, 0);
 }
 
 
 xmlrpc_mem_block *
-xmlrpc_base64_decode (xmlrpc_env *env,
-		      char *ascii_data,
-		      size_t ascii_len)
-{
+xmlrpc_base64_decode (xmlrpc_env * const env,
+                      const char * const ascii_data,
+                      size_t       const ascii_len) {
+
     unsigned char *bin_data;
     int leftbits;
     unsigned char this_ch;
@@ -203,6 +199,8 @@ xmlrpc_base64_decode (xmlrpc_env *env,
     size_t npad;
     size_t bin_len, buffer_size;
     xmlrpc_mem_block *output;
+    const char * next_char;
+    size_t remaining_len;
 
     /* Create a block to hold our chunks when we finish them.
     ** We overestimate the size now, and fix it later. */
@@ -217,50 +215,52 @@ xmlrpc_base64_decode (xmlrpc_env *env,
     bin_data = XMLRPC_TYPED_MEM_BLOCK_CONTENTS(unsigned char, output);
     bin_len = 0;
 
-    for( ; ascii_len > 0 ; ascii_len--, ascii_data++ ) {
+    for (remaining_len = ascii_len, next_char = ascii_data;
+         remaining_len > 0; 
+         --remaining_len, ++next_char) {
 
-	/* Skip some punctuation. */
-	this_ch = (*ascii_data & 0x7f);
-	if ( this_ch == '\r' || this_ch == '\n' || this_ch == ' ' )
-	    continue;
-	if ( this_ch == BASE64_PAD )
-	    npad++;
-	this_ch = table_a2b_base64[(*ascii_data) & 0x7f];
+        /* Skip some punctuation. */
+        this_ch = (*next_char & 0x7f);
+        if ( this_ch == '\r' || this_ch == '\n' || this_ch == ' ' )
+            continue;
+        if ( this_ch == BASE64_PAD )
+            npad++;
+        this_ch = table_a2b_base64[(*next_char) & 0x7f];
 
-	/* XXX - We just throw away invalid characters. Is this right? */
-	if ( this_ch == (unsigned char) -1 ) continue;
+        /* XXX - We just throw away invalid characters. Is this right? */
+        if ( this_ch == (unsigned char) -1 ) continue;
 
-	/* Shift it in on the low end, and see if there's
-	** a byte ready for output. */
-	leftchar = (leftchar << 6) | (this_ch);
-	leftbits += 6;
-	if ( leftbits >= 8 ) {
-	    leftbits -= 8;
-	    XMLRPC_ASSERT(bin_len < buffer_size);
-	    *bin_data++ = (leftchar >> leftbits) & 0xFF;
-	    leftchar &= ((1 << leftbits) - 1);
-	    bin_len++;
-	}
+        /* Shift it in on the low end, and see if there's
+        ** a byte ready for output. */
+        leftchar = (leftchar << 6) | (this_ch);
+        leftbits += 6;
+        if ( leftbits >= 8 ) {
+            leftbits -= 8;
+            XMLRPC_ASSERT(bin_len < buffer_size);
+            *bin_data++ = (leftchar >> leftbits) & 0xFF;
+            leftchar &= ((1 << leftbits) - 1);
+            bin_len++;
+        }
     }
 
     /* Check that no bits are left. */
     if ( leftbits )
-	XMLRPC_FAIL(env, XMLRPC_PARSE_ERROR, "Incorrect Base64 padding");
+        XMLRPC_FAIL(env, XMLRPC_PARSE_ERROR, "Incorrect Base64 padding");
 
     /* Check to make sure we have a sane amount of padding. */
     if (npad > bin_len || npad > 2)
-	XMLRPC_FAIL(env, XMLRPC_PARSE_ERROR, "Malformed Base64 data");
+        XMLRPC_FAIL(env, XMLRPC_PARSE_ERROR, "Malformed Base64 data");
 
     /* Remove any padding and set the correct size. */
     bin_len -= npad;
     XMLRPC_TYPED_MEM_BLOCK_RESIZE(char, env, output, bin_len);
     XMLRPC_ASSERT(!env->fault_occurred);
 
- cleanup:
+                      cleanup:
     if (env->fault_occurred) {
-	if (output)
-	    xmlrpc_mem_block_free(output);
-	return NULL;
+        if (output)
+            xmlrpc_mem_block_free(output);
+        return NULL;
     }
     return output;
 }
